﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data.SqlClient;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Data.Linq;
using GT.Comun;

namespace GT.AccesoDatos
{
    public class AreaAD : BaseAD
    {
        #region consultas

        /// <summary>
        /// Obtiene una Area por los caracteres de su nombre
        /// </summary>
        /// <param name="nombre">Nombre</param>
        /// <returns>BindingList de Area</returns>
        public BindingList<Area> ObtenerAreaPorLike(string nombre)
        {
            IQueryable<Area> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Area>(area => area.TipoArea);
            dc.LoadOptions = loadOptions;
            query = from area in dc.Area where area.Nombre.Contains(nombre) select area;
            return QueryToBindingList(query);
           
        }
        
        
        
        /// <summary>
        /// Obtiene una Area por Nombre y Tipo
        /// </summary>
        /// <param name="nombre">Nombre</param>
        /// <param name="tipo">Tipo</param>
        /// <returns>Area</returns>
        public Area ObtenerAreaPorNombreyTipo(string nombre, string tipo)
        {
            Area result;
            EntidadesDataContext dc = GetDC();

            result = dc.Area.Where(area => area.Nombre == nombre && area.Tipo == tipo).SingleOrDefault();

            return result;
        }

        /// <summary>
        /// Obtiene una Area por su identificador
        /// </summary>
        /// <param name="idArea">id del Area</param>
        /// <returns>Area</returns>
        public Area ObtenerAreaPorId(int idArea)
        {
            Area result;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Area>(area => area.TipoArea);
            dc.LoadOptions = loadOptions;
            result = dc.Area.Where(area => area.IdArea == idArea).SingleOrDefault();

            return result;
        }

        /// <summary>
        /// Obtiene todas las areas
        /// </summary>
        /// <returns>BindingList de areas</returns>
        public BindingList<Area> ObtenerAreas()
        {
            IQueryable<Area> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Area>(area => area.TipoArea);
            dc.LoadOptions = loadOptions;
            query = from area in dc.Area select area;
            return QueryToBindingList(query);
        }

        /// <summary>
        /// Obtiene todas las areas de un usuario
        /// </summary>
        /// <param name="idUsuario">id del Usuario</param>
        /// <returns>BindingList de areas</returns>
        public BindingList<Area> ObtenerAreasPorExperto(int idUsuario)
        {
            IQueryable<Area> query;
            EntidadesDataContext dc = GetDC();
            query = from usuarioArea in dc.UsuarioArea.Where(usua => usua.Aceptado == true && 
                usua.IdUsuario == idUsuario) select usuarioArea.Area;
            return QueryToBindingList(query);
        }

        /// <summary>
        /// Obtiene todas las areas que tienen usuarios pendientes de aprobar
        /// </summary>
        /// <returns>BindingList de areas</returns>
        public BindingList<Area> ObtenerAreasConUsuarioPendiente()
        {
            IQueryable<Area> query;
            EntidadesDataContext dc = GetDC();
            query = from usuarioArea in dc.UsuarioArea.Where(usua => usua.Aceptado == false) select usuarioArea.Area;
            return QueryToBindingList(query);
        }

        /// <summary>
        /// Obtiene todas las areas que tiene un usuario pendiente de aprobar
        /// </summary>
        /// <param name="idUsuario">id del Usuario</param>
        /// <returns>BindingList de areas</returns>
        public BindingList<Area> ObtenerAreasUsuarioPendiente(int idUsuario)
        {
            IQueryable<Area> query;
            EntidadesDataContext dc = GetDC();
            query = from usuarioArea in dc.UsuarioArea.Where(usua => usua.Aceptado == false && 
                usua.IdUsuario == idUsuario) select usuarioArea.Area;
            return QueryToBindingList(query);
        }
        
        #endregion



        #region mantenimientos

        /// <summary>
        /// Añade una area al sistema
        /// </summary>
        /// <param name="area">area a añadir</param>
        /// <returns>id del area agregada</returns>
        public int AgregarArea(Area area)
        {
            EntidadesDataContext dc = GetDC();
            area.TipoArea = dc.TipoArea.Where(tipoArea => tipoArea.Tipo == area.TipoAsignado).SingleOrDefault();
            dc.Area.InsertOnSubmit(area);
            dc.SubmitChanges();
            return area.IdArea;
        }

        /// <summary>
        /// Elimina una area 
        /// </summary>
        /// <param name="idArea">id del Area a Eliminar</param>
        public void EliminarArea(int idArea)
        {
            EntidadesDataContext dc = GetDC();
            //borro las relaciones con los terminos
            dc.TerminoArea.DeleteAllOnSubmit(dc.TerminoArea.Where(tera => tera.IdArea == idArea));
            //borro las relaciones con los usuarios
            dc.UsuarioArea.DeleteAllOnSubmit(dc.UsuarioArea.Where(a => a.IdArea == idArea));
            //finalmente borro el area
            Area ar = dc.Area.Where(a => a.IdArea == idArea).SingleOrDefault(); 
            dc.Area.DeleteOnSubmit(ar);
            dc.SubmitChanges();
        }

        /// <summary>
        /// Actualiza una area
        /// </summary>
        /// <param name="area">area modificada</param>
        public void ModificarArea(Area area)
        {
            EntidadesDataContext dc = GetDC();
            Area ar = dc.Area.Where(_area => _area.IdArea == area.IdArea).SingleOrDefault();
            TipoArea tipo = dc.TipoArea.Where(_tp => _tp.Tipo == area.TipoAsignado).SingleOrDefault();
            ar.TipoArea = tipo;
            ar.Nombre = area.Nombre;
            ar.Descripcion = area.Descripcion;
            dc.SubmitChanges();
        }


        #endregion


    }
}
