﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data.SqlClient;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Data.Linq;
using GT.Comun;

namespace GT.AccesoDatos
{
    public class MensajeAD : BaseAD
    {
        #region consultas

        /// <summary>
        /// Obtiene un Mensaje por su identificador
        /// </summary>
        /// <param name="idMensaje">Id del mensaje</param>
        /// <returns>Mensaje</returns>
        public Mensaje ObtenerMensajePorId(int idMensaje)
        {
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario);
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario1);
            dc.LoadOptions = loadOptions;
            return dc.Mensaje.Where(men => men.IdMensaje == idMensaje).SingleOrDefault();
          }


        /// <summary>
        /// Obtiene los Mensajes por su destinatario
        /// </summary>
        /// <param name="idDestinatario">Id del destinatario</param>
        /// <returns>BindingList de Mensaje</returns>
        public BindingList<Mensaje> ObtenerMensajePorDestinatario(int idDestinatario)
        {
            IQueryable<Mensaje> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario);
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario1);
            dc.LoadOptions = loadOptions;
            query = from mensaje in dc.Mensaje where mensaje.Destinatario == idDestinatario select mensaje;
            return QueryToBindingList(query);

        }

        /// <summary>
        /// Obtiene los Mensajes por su remitente
        /// </summary>
        /// <param name="idRemitente">Id del remitente</param>
        /// <returns>BindingList de Mensaje</returns>
        public BindingList<Mensaje> ObtenerMensajePorRemitente(int idRemitente)
        {
            IQueryable<Mensaje> query;
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario);
            loadOptions.LoadWith<Mensaje>(mensaje => mensaje.Usuario1);
            dc.LoadOptions = loadOptions;
            query = from mensaje in dc.Mensaje where mensaje.Remitente == idRemitente select mensaje;
            return QueryToBindingList(query);

        }

        #endregion

        #region actualizaciones

        /// <summary>
        /// Actualiza el contenido de un mensaje
        /// </summary>
        /// <param name="mensaje">Mensaje que se va a actualizar</param>
        public void ActualizarMensaje(Mensaje mensaje)
        {
            EntidadesDataContext dc = GetDC();
            Mensaje mensajeActualizar = dc.Mensaje.Where(mens => mens.IdMensaje == mensaje.IdMensaje).SingleOrDefault();
            mensajeActualizar.Asunto = mensaje.Asunto;
            mensajeActualizar.Descripcion = mensaje.Descripcion;
            dc.SubmitChanges();
           
        }


        /// <summary>
        /// Añade un mensaje al sistema
        /// </summary>
        /// <param name="asunto">Asunto del mensaje</param>
        /// <param name="contenido">Contenido del mensaje</param>
        /// <param name="idRemitente">Identificador del usuario que remite el mensaje</param>
        /// <param name="idDestinatario">Identificador del usuario que destinatario del mensaje</param>
        /// <returns>Identificador del Mensaje</returns>
        public int AgregarMensaje(string asunto,string contenido, int idRemitente, int idDestinatario)
        {
            EntidadesDataContext dc = GetDC();
            Mensaje mensaje = new Mensaje();
            Usuario remitente = dc.Usuario.Where(usuario => usuario.IdUsuario == idRemitente).SingleOrDefault();
            Usuario destinatario = dc.Usuario.Where(usuario => usuario.IdUsuario == idDestinatario).SingleOrDefault();
            mensaje.Asunto = asunto;
            mensaje.Descripcion = contenido;
            mensaje.Usuario = destinatario;
            mensaje.Usuario1 = remitente;
            dc.Mensaje.InsertOnSubmit(mensaje);
            dc.SubmitChanges();
            return mensaje.IdMensaje;

        }

        /// <summary>
        /// Elimina un mensaje
        /// </summary>
        /// <param name="idMensaje">Identificador del Mensaje</param>
        public void EliminarMensajePorId(int idMensaje)
        {
            
            EntidadesDataContext dc = GetDC();
            if (dc.Mensaje.Where(mens => mens.IdMensaje == idMensaje).SingleOrDefault() != null)
            {
                dc.Mensaje.DeleteOnSubmit(dc.Mensaje.Where(mens => mens.IdMensaje == idMensaje).SingleOrDefault());
                dc.SubmitChanges();
            }
        }


        #endregion

    }
}
