﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data.SqlClient;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Data.Linq;
using GT.Comun;

namespace GT.AccesoDatos
{
    public class TipoAreaAD : BaseAD
    {
        #region consultas

        /// <summary>
        /// Obtiene los tipos de area tematica
        /// </summary>
        /// <returns>Lista de areas tematicas</returns>
        public BindingList<TipoArea> ObtenerTiposArea()
        {
            IQueryable<TipoArea> query;
            EntidadesDataContext dc = GetDC();

            query = from tipoArea in dc.TipoArea select tipoArea;
            return QueryToBindingList(query);
        }

        /// <summary>
        /// Obtiene los tipos de area por un like de su nombre
        /// </summary>
        /// <param name="nombre">Nombre del Tipo de Area</param>
        /// <returns>BindingList de TipoArea</returns>
        public BindingList<TipoArea> ObtenerTipoAreaPorLike(string nombre)
        {
            IQueryable<TipoArea> query;
            EntidadesDataContext dc = GetDC();
            query = from area in dc.TipoArea where area.Tipo.Contains(nombre) select area;
            return QueryToBindingList(query);

        }

        #endregion



        #region mantenimientos
        /// <summary>
        /// Añade un tipo de area al sistema
        /// </summary>
        /// <param name="tipoArea">tipoArea a añadir</param>
        public void AgregarTipoArea(TipoArea tipoArea)
        {
            EntidadesDataContext dc = GetDC();
            dc.TipoArea.InsertOnSubmit(tipoArea);
            dc.SubmitChanges();
        }

        /// <summary>
        /// Elimina un tipo de area 
        /// </summary>
        /// <param name="tipoArea">Tipo de Area a Eliminar</param>
        public void EliminarTipoArea(TipoArea tipoArea)
        {
            EntidadesDataContext dc = GetDC();
            TipoArea ar = dc.TipoArea.Where(tarea => tarea.Tipo == tipoArea.Tipo).SingleOrDefault();
            dc.TipoArea.DeleteOnSubmit(ar);
            dc.SubmitChanges();
        }

        /// <summary>
        /// Actualiza un tipo de area
        /// </summary>
        /// <param name="tipoAreaNew">Tipo de Area nueva</param>
        /// <param name="tipoAreaOld">Tipo de Area a modificar</param>
        public void ModificarTipoArea(TipoArea tipoAreaNew, TipoArea tipoAreaOld)
        {
            EntidadesDataContext dc = GetDC();
            TipoArea ar = dc.TipoArea.Where(tarea => tarea.Tipo == tipoAreaOld.Tipo).SingleOrDefault();
            ar.Tipo = tipoAreaNew.Tipo;
            ar.Descripcion = tipoAreaNew.Descripcion;
            dc.SubmitChanges();
        }

        #endregion

    }
}
