﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Data.SqlClient;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Data.Linq;
using GT.Comun;
using GT.Comun.Helpers;

namespace GT.AccesoDatos
{
    public class UsuarioAD : BaseAD
    {

        #region actualizaciones

        /// <summary>
        /// Elimina un usuario en el sistema
        /// </summary>
        /// <param name="usuario">Usuario a añadir</param>
        public void EliminarUsuario(int idUsuario)
        {
            EntidadesDataContext dc = GetDC();
            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Usuario>(usuario => usuario.UsuarioArea);
            loadOptions.LoadWith<Usuario>(usuario => usuario.Mensaje);
            loadOptions.LoadWith<Usuario>(usuario => usuario.Mensaje1);
            dc.LoadOptions = loadOptions;
            Usuario user = dc.Usuario.Where(usuar =>
                usuar.IdUsuario == idUsuario).SingleOrDefault();
            //Borro todas los mensajes del usuario
            user.Mensaje.Clear();
            user.Mensaje1.Clear();
            //Borro las areas relacionadas
            dc.UsuarioArea.DeleteOnSubmit(dc.UsuarioArea.Where(a => a.IdUsuario==idUsuario).SingleOrDefault());
            //Borro al usuario
            dc.Usuario.DeleteOnSubmit(user);

            dc.SubmitChanges();
        }

        /// <summary>
        /// Modificar un usuario en el sistema
        /// </summary>
        /// <param name="usuario">Usuario a añadir</param>
        /// <returns>Id del usuario</returns>
        public int ModificarUsuario(Usuario usuario)
        {
            EntidadesDataContext dc = GetDC();
            Usuario user = dc.Usuario.Where(usuar => 
                usuar.IdUsuario == usuario.IdUsuario).SingleOrDefault();
            user.Email = usuario.Email;
            user.Nombre = usuario.Nombre;
            user.Rol = usuario.Rol;
            dc.SubmitChanges();
            return usuario.IdUsuario;
        }

        /// <summary>
        /// Añade un usuario al sistema
        /// </summary>
        /// <param name="usuario">Usuario a añadir</param>
        /// <returns>Id del usuario</returns>
        public int AgregarUsuario(Usuario usuario)
        {
            EntidadesDataContext dc = GetDC();
            dc.Usuario.InsertOnSubmit(usuario);
            dc.SubmitChanges();
            return usuario.IdUsuario;
        }


        /// <summary>
        /// Añade un usuario a un area
        /// </summary>
        /// <param name="usuario">Id del Usuario</param>
        /// <param name="idArea">Id del Area</param>
        public void AgregarUsuarioToArea(int idUsuario, int idArea)
        {
            EntidadesDataContext dc = GetDC();
            if (dc.UsuarioArea.Where(a => a.Usuario.IdUsuario == idUsuario &&
                a.IdArea == idArea).SingleOrDefault() == null)
            {
                Usuario usuario = dc.Usuario.Where(usu => usu.IdUsuario == idUsuario).SingleOrDefault();
                Area area = dc.Area.Where(are => are.IdArea == idArea).SingleOrDefault();
                UsuarioArea newUsuario = new UsuarioArea();
                newUsuario.Usuario = usuario;
                newUsuario.Area = area;
                dc.UsuarioArea.InsertOnSubmit(newUsuario);
                dc.SubmitChanges();
            }
            else
            {
                throw new Exception(ErrorConfig.Negocio + " - Solicitud previamente anotada");
            }
            
        }
        
        /// <summary>
        /// Aceptar una solicitud de experto en area
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        /// <param name="idArea">Id del area</param>
        public void AceptarUsuarioEnArea(int idUsuario, int idArea)
        {
            EntidadesDataContext dc = GetDC();
            UsuarioArea usuarea = dc.UsuarioArea.Where(usu => usu.IdUsuario == idUsuario 
                && usu.IdArea == idArea).SingleOrDefault();
            usuarea.Aceptado = true;
            dc.SubmitChanges();
        }


        /// <summary>
        /// Rechazar una solicitud de experto en area
        /// </summary>
        /// <param name="idUsuario">Id del usuario</param>
        /// <param name="idArea">Id del area</param>
        public void EliminarUsuarioEnArea(int idUsuario, int idArea)
        {
            EntidadesDataContext dc = GetDC();
            UsuarioArea usuarea = dc.UsuarioArea.Where(usu => usu.IdUsuario == idUsuario
                && usu.IdArea == idArea).SingleOrDefault();
            dc.UsuarioArea.DeleteOnSubmit(usuarea);
            dc.SubmitChanges();
        }

        #endregion



        #region Consultas

        /// <summary>
        /// Obtiene todos los usuarios
        /// </summary>
        /// <returns>BindingList de Usuario</returns>
        public BindingList<Usuario> ObtenerUsuarios()
        {
            IQueryable<Usuario> query;
            EntidadesDataContext dc = GetDC();
            query = from usuario in dc.Usuario select usuario;
            return QueryToBindingList(query);
        }

        /// <summary>
        /// Obtiene todos los usuarios que cumplen ciertos criterios
        /// </summary>
        /// <param name="nombre">Nombre del usuario</param>
        /// <param name="email">Email del usuario</param>
        /// <param name="rol">Rol del usuario</param>
        /// <returns>BindingList de Usuario</returns>
        public BindingList<Usuario> ObtenerUsuariosFiltradoPor(String nombre, String email, Nullable<Rol> rol)
        {
            IQueryable<Usuario> query;
            EntidadesDataContext dc = GetDC();
            query = from usuario in dc.Usuario select usuario;
            if (nombre != "")
                query = query.Where(usuario => usuario.Nombre.Contains(nombre));
            if (email != "")
                query = query.Where(usuario => usuario.Email.Contains(email));
            if (rol.HasValue)
                query = query.Where(usuario => usuario.Rol == rol.Value);
            return QueryToBindingList(query);
        }


        /// <summary>
         /// Obtiene un usuario por su nombre
         /// </summary>
        /// <param name="nombre">Nombre del usuario</param>
         /// <returns>Usuario</returns>
         public Usuario ObtenerUsuarioPorNombre(string nombre)
         {
             Usuario result;
             EntidadesDataContext dc = GetDC();

             result = dc.Usuario.Where(usuario => usuario.Nombre == nombre).SingleOrDefault();

             return result;
         }


         /// <summary>
         /// Obtiene un usuario por su identificador
         /// </summary>
         /// <param name="idUsuario">Id del usuario</param>
         /// <returns>Usuario</returns>
         public Usuario ObtenerUsuarioPorId(int idUsuario)
         {
             Usuario result;
             EntidadesDataContext dc = GetDC();

             result = dc.Usuario.Where(usuario => usuario.IdUsuario == idUsuario).SingleOrDefault();

             return result;
         }

         /// <summary>
         /// Obtiene los usuarios con solicitud pendiente
         /// </summary>
         /// <returns>BindingList de Usuario</returns>
         public BindingList<Usuario> ObtenerUsuariosSolicitudPendiente()
         {
             IQueryable<Usuario> query;
             EntidadesDataContext dc = GetDC();
             query = from usuarioArea in dc.UsuarioArea.Where(usua => usua.Aceptado == false) select usuarioArea.Usuario;
             return QueryToBindingList(query);
         }

         /// <summary>
         /// Obtiene los usuarios con solicitud pendiente en un area
         /// </summary>
         /// <param name="idArea">Id del area</param>
         /// <returns>BindingList de Usuario</returns>
         public BindingList<Usuario> ObtenerUsuariosSolicitudPendiente(int idArea)
         {
             IQueryable<Usuario> query;
             EntidadesDataContext dc = GetDC();
             query = from usuarioArea in dc.UsuarioArea.Where(usua => usua.Aceptado == false && usua.IdArea == idArea) 
                     select usuarioArea.Usuario;
             return QueryToBindingList(query);
         }

         /// <summary>
         /// Obtiene los usuarios con areas comunes
         /// </summary>
         /// <param name="idUsuario">Id del usuario</param>
         /// <returns>Usuario</returns>
         public BindingList<Usuario> ObtenerUsuariosPorAreaComun(int idUsuario)
         {
             EntidadesDataContext dc = GetDC();
             BindingList<Usuario> resultado = new BindingList<Usuario>();
             using (SqlConnection cnn = (SqlConnection)dc.Connection)
             {
                 SqlCommand cmd = new SqlCommand("ObtenerUsuariosAreaComunPorUsuarioId", cnn);
                 cmd.CommandType = CommandType.StoredProcedure;
                 cmd.Parameters.AddWithValue("idUsuario", idUsuario);

                 cnn.Open();
                 SqlDataReader reader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                 while (reader.Read())
                 {
                     Usuario itm = new Usuario();
                     itm.IdUsuario = (int)reader["IdUsuario"];
                     itm.Nombre = (string)reader["Nombre"];
                     itm.Email = (string)reader["Email"];
                     itm.Rol = (Rol)reader["Rol"];
                     resultado.Add(itm);
                 }
                 reader.Close();
                 foreach (Usuario itm in resultado)
                 {
                     itm.UsuarioArea.AddRange(dc.UsuarioArea.Where(a => a.IdUsuario == itm.IdUsuario));
                 }
             }
             return resultado;
         }

        #endregion
         
         #region "Operaciones sobre la clave"

        /// <summary>
        /// Obtiene la clave cifrada del usuario dado
        /// </summary>
        /// <param name="nombre">nombre del usuario</param>
        /// <returns>Clave cifrada</returns>
        public string ObtenerClave(string nombre)
        {
            EntidadesDataContext dc = GetDC();

            using (SqlConnection cnn = (SqlConnection)dc.Connection)
            {
                SqlCommand cmd = new SqlCommand("ObtenerClaveUsuario", cnn);
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.AddWithValue("nombre", nombre);

                cnn.Open();
                SqlDataReader reader = cmd.ExecuteReader();
                if (reader.Read())
                    return (string)reader["Clave"];
                else
                    return null;
            }
        }

        #endregion


    }
}
