﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GT.Comun;
using GT.Comun.Helpers;
using GT.AccesoDatos;
using System.ComponentModel;
using System.Security.Principal;
using System.Threading;
using System.Security.Permissions;
namespace GT.LogicaNegocio
{
    public class MensajeCN
    {
        /// <summary>
        /// Modifica un mensaje
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="mensaje">Mensaje</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public void ActualizarMensaje(Mensaje mensaje)
        {
            MensajeAD mensajeAD = new MensajeAD();
            mensajeAD.ActualizarMensaje(mensaje);
        }
        
        /// <summary>
        /// Obtiene un mensaje por su identificador
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idMensaje">Id del Mensaje</param>
        /// <returns>Mensaje</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public Mensaje ObtenerMensajePorId(int idMensaje)
        {
            MensajeAD mensajeAD = new MensajeAD();
            return mensajeAD.ObtenerMensajePorId(idMensaje);
        }


        /// <summary>
        /// Elimina un Mensaje
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idMensaje">Id del Mensaje</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public void EliminarMensaje(int idMensaje)
        {
            MensajeAD mensajeAD = new MensajeAD();
            mensajeAD.EliminarMensajePorId(idMensaje);
        }
        
        
        /// <summary>
        /// Añade un mensaje al sistema
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="asunto">Asunto</param>
        /// <param name="contenido">Contenido</param>
        /// <param name="idRemitente">Id del usuario remitente</param>
        /// <param name="idDestinatario">Id del usuario destinatario</param>
        /// <returns>Id del mensaje</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public int AgregarMensaje(string asunto, string contenido, int idRemitente, int idDestinatario)
        {
            MensajeAD mensajeAD = new MensajeAD();
            return mensajeAD.AgregarMensaje(asunto, contenido, idRemitente, idDestinatario);
        }

        /// <summary>
        /// Obtiene los mensajes por su destinatario
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idDestinatario">Id del usuario destinatario</param>
        /// <returns>BindingList de Mensaje</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Mensaje> ObtenerMensajePorDestinatario(int idDestinatario)
        {
            MensajeAD mensajeAD = new MensajeAD();
            return mensajeAD.ObtenerMensajePorDestinatario(idDestinatario);
        }

        /// <summary>
        /// Obtiene los mensajes por su remitente
        /// Seguridad: Administrador, Experto
        /// </summary>
        /// <param name="idRemitente">Id del usuario remitente</param>
        /// <returns>BindingList de Mensaje</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public BindingList<Mensaje> ObtenerMensajePorRemitente(int idRemitente)
        {
            MensajeAD mensajeAD = new MensajeAD();
            return mensajeAD.ObtenerMensajePorRemitente(idRemitente);
        }

    }
}
