﻿using System;
using System.Data.Common;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GT.Comun;
using GT.Comun.Helpers;
using GT.AccesoDatos;
using System.ComponentModel;
using System.Security.Principal;
using System.Threading;
using System.Security.Permissions;
using System.Transactions;

namespace GT.LogicaNegocio
{
   public class TerminoCN
    {
        #region Consulta

        /// <summary>
        /// Obtiene los terminos por su area
        /// Seguridad: Publico
        /// </summary>
        /// <param name="idArea">Id del area</param>
        /// <returns>BindingList de Termino</returns>
       public BindingList<Termino> ObtenerTerminosPorAreaId(int idArea) {
           TerminoAD terminoAD = new TerminoAD();
           return terminoAD.ObtenerTerminosPorAreaId(idArea);
       
       }

       /// <summary>
       /// Obtiene los terminos por un like del noimbre
       /// Seguridad: Publico
       /// </summary>
       /// <param name="nombre">Nombre</param>
       /// <returns>BindingList de Termino</returns>
       public BindingList<Termino> ObtenerTerminosPorLike(String nombre)
       {
           TerminoAD terminoAD = new TerminoAD();
           return terminoAD.ObtenerTerminosPorLike(nombre);

       }


       /// <summary>
       /// Obtiene un termino por su identificador
       /// Seguridad: Publico
       /// </summary>
       /// <param name="idTermino">Id del termino</param>
       /// <returns>Termino</returns>
       public Termino ObtenerTerminoPorId(int idTermino)
       {
           TerminoAD terminoAD = new TerminoAD();
           return terminoAD.ObtenerTerminoPorId(idTermino);

       }



       /// <summary>
       /// Obtiene todos los terminos del sistema
       /// Seguridad: Publico
       /// </summary>
       /// <returns>BindingList de Termino</returns>
       public BindingList<Termino> ObtenerTerminos()
       {
           TerminoAD terminoAD = new TerminoAD();
           return terminoAD.ObtenerTerminos();

       }
        
        #endregion

        #region Mantenimientos



       /// <summary>
       /// Elimina un termino en una serie de areas
       /// Seguridad: Administrador, Experto
       /// </summary>
       /// <param name="idTermino">Id del termino</param>
       /// <param name="idArea">Lista de areas</param>
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
       public void eliminarTermino(int idTermino, List<int> idArea)
       {

           TerminoAD terminoAD = new TerminoAD();
           using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope())
           {
               terminoAD.EliminarTermino(idTermino);
               foreach (int id in idArea)
                   terminoAD.DesAsignarTerminoToArea(idTermino, id);
               trans.Complete();
           }
       }

       /// <summary>
       /// Modifica un termino en una serie de areas
       /// Seguridad: Administrador, Experto
       /// </summary>
       /// <param name="termino">Termino</param>
       /// <param name="idArea">Lista de areas</param>
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
       public void modificarTermino(Termino termino, List<int> idArea)
       {
        try
            {
           
               TerminoAD terminoAD = new TerminoAD();
               using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope())
               {
                  terminoAD.DesAsignarTerminoToAreas(termino.IdTermino);
                  terminoAD.ModificarTermino(termino);
                  foreach (int id in idArea)
                       terminoAD.AsignarTerminoToArea(termino.IdTermino, id);
                  trans.Complete();
               }
            }
        catch (Exception ex)
        {

            throw ex; //solo está para poder detenerse y depurar
        }
       }

       /// <summary>
       /// Añade un termino en una serie de areas
       /// Seguridad: Administrador, Experto
       /// </summary>
       /// <param name="termino">Termino</param>
       /// <param name="idArea">Lista de areas</param>
       /// <returns>Id del termino (0 en caso de error)</returns>
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
       [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Experto")]
        public int agregarTermino(Termino termino, List<int> idArea)
        {

            try
            {
                TerminoAD terminoAD = new TerminoAD();
                using (System.Transactions.TransactionScope trans = new System.Transactions.TransactionScope(TransactionScopeOption.Required,TimeSpan.Zero))
                {
                    int res = terminoAD.AgregarTermino(termino);
                    foreach (int id in idArea)
                        terminoAD.AsignarTerminoToArea(termino.IdTermino, id);
                    trans.Complete();
                    return res;
                }
            
            }
            catch (Exception) 
            {
                
                return 0;
            }
            
            
        }

        #endregion
    }
}
