﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GT.Comun;
using GT.Comun.Helpers;
using GT.AccesoDatos;
using System.ComponentModel;
using System.Security.Principal;
using System.Threading;
using System.Security.Permissions;

namespace GT.LogicaNegocio
{
    public class TipoAreaCN
    {

        #region consulta

        /// <summary>
        /// Obtiene los tipos de area
        /// Seguridad: Administrador
        /// </summary>
        /// <returns>BindingList de tipos de area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<TipoArea> ObtenerTiposArea()
        {
            TipoAreaAD tipoAreaAD = new TipoAreaAD();

            return tipoAreaAD.ObtenerTiposArea();
        }

        /// <summary>
        /// Obtiene los tipos de area por un like del nombre
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="nombre">nombre</param>
        /// <returns>BindingList de tipos de area</returns>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public BindingList<TipoArea> ObtenerTipoAreaPorLike(string nombre)
        {
            TipoAreaAD tipoAreaAD = new TipoAreaAD();

            return tipoAreaAD.ObtenerTipoAreaPorLike(nombre);
        }

        #endregion


        #region mantenimientos

        /// <summary>
        /// Añade un tipo de area al sistema
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="tipoarea">Tipo de area</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void AgregarTipoArea(TipoArea tipoarea)
        {
            TipoAreaAD tipoAreaAD = new TipoAreaAD();
            tipoAreaAD.AgregarTipoArea(tipoarea);

        }

        /// <summary>
        /// Modifica un tipo de area
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="tipoAreaNew">Area modificada</param>
        /// <param name="tipoAreaOld">Area original</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void ModificarTipoArea(TipoArea tipoAreaNew, TipoArea tipoAreaOld)
        {
            TipoAreaAD tipoAreaAD = new TipoAreaAD();
            tipoAreaAD.ModificarTipoArea(tipoAreaNew, tipoAreaOld);

        }

        /// <summary>
        /// Elimina un tipo de area
        /// Seguridad: Administrador
        /// </summary>
        /// <param name="tipoarea">tipo de area</param>
        [PrincipalPermission(SecurityAction.Demand, Authenticated = true, Role = "Administrador")]
        public void EliminarTipoArea(TipoArea tipoarea)
        {
            TipoAreaAD tipoAreaAD = new TipoAreaAD();
            tipoAreaAD.EliminarTipoArea(tipoarea);

        }

        #endregion
    }
}
