﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ServiceModel;
using GT.Comun;
using System.ServiceModel.Description;
using System.ComponentModel;
using System.Web.Security;
using GT.Web.GTServicios;

namespace GT.Web
{
    public class ControladorServicio : IDisposable
    {
        private GTServiciosClient gtservicio;
        private string kInvitado = "invitado";

               
        public ControladorServicio(bool invitado)
        {
            if (invitado)
            {
                gtservicio = new GTServiciosClient("WSHttpBinding_IGTServicios");
                gtservicio.ClientCredentials.UserName.UserName = kInvitado;
            }
            else
            {
                try
                {
                    gtservicio = new GTServiciosClient("WSHttpBinding_IGTServicios");
                    gtservicio.ClientCredentials.UserName.UserName = HttpContext.Current.User.Identity.Name;
                    gtservicio.ClientCredentials.UserName.Password = (String)HttpContext.Current.Session["Clave"];
                    
                }
                catch 
                {
                    //Redirigimos a la pagina de login
                    HttpContext.Current.Server.Transfer(HttpContext.Current.Request.ServerVariables["HTTP_HOST"] + "/login.aspx");
                    
                }
                
                
            }

        }
        #region Usuario

        public Boolean EsExpertoAreas(string nombre)
        {
            return gtservicio.Usuarios_EsExperto(nombre);
        }


        public int AgregarUsuario(Usuario usuario, out string errorMsg)
        {
            try{
                errorMsg = null;
                return gtservicio.Usuarios_AgregarUsuario(usuario); ;

            }catch (Exception ex){
        
                errorMsg = ex.Message;
                return 0;
            
            }
    
        }
        public void AgregarUsuarioToArea(int idUsuario, int idArea)
        {

            gtservicio.Usuarios_AgregarUsuarioToArea(idUsuario, idArea);
        }
        public bool ValidarUsuario(string nombre,string clave, out Usuario usuario)
        {
            
            usuario = gtservicio.Usuarios_ValidarUsuario(nombre, clave);
            if (usuario == null)
            {
                return false;
            }
            else
            {
                HttpContext.Current.Session["Clave"] = clave;
                return true;
            }
           
        }

        public string ObtenerRolUsuario(string nombre)
        {
            string res = null;
            
            try
            {
                res = gtservicio.Usuarios_ObtenerRolUsuario(nombre).ToString();
            }
            catch  {
                res = "invitado";
            }

            return res;

        }
        public Usuario ObtenerUsuarioPorId(int idUsuario)
        {

            return gtservicio.Usuarios_ObtenerUsuarioPorId(idUsuario);

        }
        
        
        public Usuario ObtenerUsuarioPorNombre(String nombre)
        {

            return gtservicio.Usuarios_ObtenerUsuarioPorNombre(nombre);

        }
        public BindingList<Usuario> ObtenerUsuariosPendientes()
        {

            return gtservicio.Usuarios_ObtenerUsuariosSolicitudPendiente();

        }
        public BindingList<Usuario> ObtenerUsuariosFiltradoPor(String nombre, String email,Nullable<Rol> rol)
        {

            return gtservicio.Usuarios_ObtenerUsuariosFiltradoPor(nombre, email, rol);

        }
        public BindingList<Usuario> ObtenerUsuariosPendientes(int idArea)
        {

            return gtservicio.Usuarios_ObtenerUsuariosSolicitudPendiente();

        }
        public BindingList<Usuario> ObtenerUsuariosPorAreaComun(String nombre)
        {

            return gtservicio.Usuarios_ObtenerUsuariosPorAreaComun(nombre);

        }

        public BindingList<Usuario> ObtenerUsuarios()
        {

            return gtservicio.Usuarios_ObtenerUsuarios();

        }

        public void AceptarUsuarioEnArea(int idUsuario,BindingList<int> idsAreas)
        {

            gtservicio.Usuarios_AceptarUsuarioEnArea(idUsuario,idsAreas);

        }
        public void EliminarUsuarioEnArea(int idUsuario, BindingList<int> idsAreas)
        {

            gtservicio.Usuarios_EliminarUsuarioEnArea(idUsuario, idsAreas);

        }

        public void ActualizarUsuario(Usuario usuario, BindingList<int> idsAreas)
        {
            gtservicio.Usuarios_ActualizarUsuario(usuario, idsAreas);
        }

        public void EliminarUsuario(int idUsuario)
        {
            gtservicio.Usuarios_EliminarUsuario(idUsuario);
        }

        #endregion

        #region TipoArea

        public void AgregarTipoArea(TipoArea tipoArea)
        {

            gtservicio.TipoAreas_AgregarTipoArea(tipoArea);
           
        }

        public void EliminarTipoArea(TipoArea tipoArea)
        {

            gtservicio.TipoAreas_EliminarTipoArea(tipoArea);

        }

        public void ModificarTipoArea(TipoArea tipoAreaNew, TipoArea tipoAreaOld)
        {

            gtservicio.TipoAreas_ModificarTipoArea(tipoAreaNew,tipoAreaOld);

        }

        public BindingList<TipoArea> ObtenerTiposArea() {

            return gtservicio.TipoAreas_ObtenerTiposArea();

        }

        public BindingList<TipoArea> ObtenerTiposArea(String nombre)
        {

            return gtservicio.TipoAreas_ObtenerTipoAreaPorLike(nombre);

        }

        #endregion

        #region Area

        public int AgregarArea(Area area)
        {
            return gtservicio.Areas_AgregarArea(area);
        }
        public BindingList<Area> ObtenerAreas()
        {
            return gtservicio.Areas_ObtenerAreas();
        }
        public BindingList<Area> ObtenerAreasConUsuarioPendiente()
        {
            return gtservicio.Areas_ObtenerAreasConUsuarioPendiente();
        }
        public BindingList<Area> ObtenerAreasUsuarioPendiente(String nombre)
        {
            return gtservicio.Areas_ObtenerAreasUsuarioPendiente(nombre);
        }
        public BindingList<Area> ObtenerAreasIdUsuarioPendiente(int idUsuario)
        {
            return gtservicio.Areas_ObtenerAreasPorIdUsuarioPendiente(idUsuario);
        }
        
        public BindingList<Area> ObtenerAreas(String nombre)
        {
            return gtservicio.Areas_ObtenerAreasPorNombre(nombre);
        }
        public Area obteneraAreaPorId(int idArea)
        {
            return gtservicio.Areas_ObtenerAreaPorId(idArea);
        }
        public BindingList<Area> obteneraAreaPorUsuarioExperto(int idAUsuario)
        {
            return gtservicio.Areas_ObtenerAreasPorExperto(idAUsuario);
        }
        
        public void EliminarArea(int idArea) {

            gtservicio.Areas_EliminarArea(idArea);
        }
        public void ModificarArea(Area area)
        {
            gtservicio.Areas_ModificarArea(area);

        }
        public void ObtenerInfoAreasUsuarioId(ref BindingList<Area> todas,
           ref BindingList<Area> solicitadas, ref BindingList<Area> experto, int idUsuario)
        {
            gtservicio.Areas_ObtenerInfoAreasUsuarioId(ref todas, ref solicitadas,ref  experto, idUsuario);
        }

        #endregion

        #region Termino

        public void modificarTermino(Termino termino, BindingList<int> idArea)
        {
            gtservicio.Terminos_ModificarTermino(termino, idArea);
        }
        public int agregarTermino(Termino termino,BindingList<int> idArea)
        {
            return gtservicio.Terminos_Agregar(termino, idArea);
        }
        public BindingList<Termino> ObtenerTerminosPorAreaId(int idArea)
        {
            return gtservicio.Terminos_ObtenerTerminosPorAreaId(idArea);
        }
        public BindingList<Termino> ObtenerTerminosPorLike(String nombre)
        {
            return gtservicio.Terminos_ObtenerTerminosPorLike(nombre);
        }
        public Termino ObtenerTerminoPorId(int idTermino)
        {
            return gtservicio.Terminos_ObtenerTerminoPorId(idTermino);
        }
        public BindingList<Termino> ObtenerTerminos()
        {
            return gtservicio.Terminos_ObtenerTerminos();
        }
        public void EliminarTerminoPorId(int idTermino, BindingList<int> areas)
        {
            gtservicio.Terminos_eliminarTermino(idTermino, areas);
        }
        #endregion

        #region Mensaje

        public int AgregarMensaje(string asunto, string contenido, int idRemitente, int idDestinatario)
        {
            return gtservicio.Mensajes_AgregarMensaje(asunto, contenido, idRemitente, idDestinatario);
        }
        public BindingList<Mensaje> ObtenerMensajePorDestinatario(int idDestinatario)
        {
            return gtservicio.Mensajes_ObtenerMensajePorDestinatario(idDestinatario);
        }
        public BindingList<Mensaje> ObtenerMensajePorRemitente(int idRemitente)
        {
            return gtservicio.Mensajes_ObtenerMensajePorRemitente(idRemitente);
        }
        public void EliminarMensaje(int idMensaje)
        {
            gtservicio.Mensajes_EliminarMensaje(idMensaje);
        }
        public void ActualizarMensaje(Mensaje mensaje) 
        {
            gtservicio.Mensajes_ActualizarMensaje(mensaje);
        }
        public Mensaje ObtenerMensajePorId(int idMensaje)
        {
            return gtservicio.Mensajes_ObtenerMensajePorId(idMensaje);
        }

        #endregion

        #region informes

        public byte[] ObtenerInforme(String informe)
        {
            return gtservicio.ObtenerInforme(informe);
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            if (gtservicio != null) { gtservicio.Close(); gtservicio = null; }
        }

        #endregion
        
    }
}
