﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using GT.Comun;
using GT.Comun.Helpers;

namespace GT.Web.Secure
{
    public partial class EnvioMensaje : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                ControladorServicio controlador = new ControladorServicio(false);
                if (Request.QueryString["Id"] != null)
                {
                    int idUsuario = Convert.ToInt32(Request.QueryString["Id"]);
                    Usuario  usuario = controlador.ObtenerUsuarioPorId(idUsuario);
                    lblDestinatario.Text =  usuario.Nombre + " - " + usuario.Email ;
                    ViewState["idDestinatario"] = idUsuario;
                }
                else
                {
                    if (Request.QueryString["IdMsg"] != null)
                    {
                        int idMensaje = Convert.ToInt32(Request.QueryString["IdMsg"]);
                        Mensaje mensaje = controlador.ObtenerMensajePorId(idMensaje);
                        lblDestinatario.Text = mensaje.NombreDestinatario;
                        ViewState["idMensaje"] = idMensaje;
                        this.txtAsunto.Text = mensaje.Asunto;
                        this.txtDescripcion.Text = mensaje.Descripcion;
                        this.btnAceptar.Text = "Actualizar";
                    }
                    else
                    {
                        BindingList<Usuario> lista;
                        if (User.IsInRole("Administrador"))
                            lista = controlador.ObtenerUsuarios();
                        else
                            lista = controlador.ObtenerUsuariosPorAreaComun(User.Identity.Name);

                        this.listaDestinatario.DataSource = lista;
                        this.listaDestinatario.DataTextField = "Nombre";
                        this.listaDestinatario.DataValueField = "IdUsuario";
                        this.listaDestinatario.DataBind();
                    }
                }
                controlador.Dispose();
            }
        }

        protected void Aceptar_Click(object sender, EventArgs e)
        {
            bool validado=false;
            ControladorServicio controlador = new ControladorServicio(false);
            if (txtAsunto.Text=="")
                ErrorMsg.Text="El asunto es obligatorio";
            else if (txtDescripcion.Text == "")
                ErrorMsg.Text = "El contenido del mensaje es obligatorio";
            else
            {
                if (ViewState["idDestinatario"] == null)
                   ViewState["idDestinatario"] = this.listaDestinatario.SelectedValue;
                
                validado = true;
            }
            if (validado)
            {
                try
                {
                    if (this.btnAceptar.Text == "Actualizar")
                    {
                        Mensaje mensaje = new Mensaje();
                        mensaje.Asunto = txtAsunto.Text;
                        mensaje.Descripcion = txtDescripcion.Text;
                        mensaje.IdMensaje = (int)Convert.ToInt32(ViewState["idDestinatario"]);
                        controlador.ActualizarMensaje(mensaje);
                    }
                    else
                    {
                        int idRemitente = controlador.ObtenerUsuarioPorNombre(Context.User.Identity.Name).IdUsuario;
                        controlador.AgregarMensaje(txtAsunto.Text, txtDescripcion.Text, idRemitente, (int)Convert.ToInt32(ViewState["idDestinatario"]));
                        CorrectoMsg.Visible = true;
                    }
                    
                }
                catch (Exception ex)
                {
                    Trace.Write(ErrorConfig.ErrorAplicacion, ex.Message, ex);
                    ErrorMsg.Visible = true;
                    ErrorMsg.Text = "Ocurrió un error enviando el mensaje";
                }
               
            }
            else
            {
                ErrorMsg.Visible = true;
            }
            controlador.Dispose();
        }
    }
}
