﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using GT.Comun;
using GT.Comun.Helpers;

namespace GT.Web
{
    public partial class TerminosListado : System.Web.UI.Page
    {
        public int idTermino;
        public String vengoDe;
        protected void Page_Load(object sender, EventArgs e)
        {
            MsgError.Visible = false;
            Session["idTermino"] = null;
            if (!IsPostBack)
            {
                try
                {
                    ControladorServicio controlador = new ControladorServicio(true);
                    BindingList<Termino> terminos;
                    if (PreviousPage != null)
                    {
                        if (PreviousPage.GetType().BaseType == typeof(AreaListado))
                        {
                            AreaListado paginaPrevia = (AreaListado)PreviousPage.Page;
                            vengoDe = "Area";
                            int idArea = paginaPrevia.idAreaSeleccionda;
                            Session["idArea"] = idArea;
                            terminos = controlador.ObtenerTerminosPorAreaId(idArea);
                        }
                        else if (PreviousPage.GetType().BaseType == typeof(TerminoConsulta))
                        {
                            TerminoConsulta paginaPrevia = (TerminoConsulta)PreviousPage.Page;
                            vengoDe = "Termino";
                            if (paginaPrevia.terminoBusqueda == "")
                                terminos = controlador.ObtenerTerminos();
                            else
                                terminos = controlador.ObtenerTerminosPorLike(paginaPrevia.terminoBusqueda);

                            this.txtTermino.Text = paginaPrevia.terminoBusqueda;
                            Session["UltimaBusqueda"] = this.txtTermino.Text;
                        }
                        else
                        {
                            terminos = controlador.ObtenerTerminos();
                        }
                    }
                    else
                    {
                        if (Session["idArea"] != null)
                        {
                            vengoDe = "Area";
                            int idArea = Convert.ToInt32(Session["idArea"]);
                            terminos = controlador.ObtenerTerminosPorAreaId(idArea);
                        }
                        else
                        {
                            vengoDe = "Termino";
                            if ((String)Session["UltimaBusqueda"] == "")
                                terminos = controlador.ObtenerTerminos();
                            else
                            {
                                terminos = controlador.ObtenerTerminosPorLike((String)Session["UltimaBusqueda"]);
                                this.txtTermino.Text = (String)Session["UltimaBusqueda"];
                            }
                        }
                    }

                    evaluarVisibles(terminos, controlador);
                    //libero los recursos
                    controlador.Dispose();
                    //Guardo la navegación a nivel de pagina
                    ViewState["vengoDe"] = vengoDe;
                }
                catch (Exception ex)
                {
                    Trace.Write(ErrorConfig.ErrorAplicacion, ex.Message, ex);
                    MsgError.Visible = true;
                }
                
             }
        }

        protected void lnkModificar_onCommand(object sender, CommandEventArgs e)
        {
            idTermino = Convert.ToInt32(e.CommandArgument);
            Server.Transfer("TerminoDetalle.aspx");
        }

        private void evaluarVisibles(BindingList<Termino> terminos, ControladorServicio controlador)
        {
            //Compruebo si he recuperado algún registro
            if (terminos.Count == 0)
            {
                lblNohayRegistros.Visible = true;
                ListaTerminos.Visible = false;
            }
            else
            {

                //muestro las areas concatenadas a las que pertenece el termino
                foreach (Termino term in terminos)
                {
                    foreach (TerminoArea tera in term.TerminoArea)
                    {
                        Area area = controlador.obteneraAreaPorId(tera.IdArea);
                        term.AreasConcatenadas = term.AreasConcatenadas + " - " + area.Nombre;
                    }
                }


                ListaTerminos.DataSource = terminos;
                ListaTerminos.Visible = true;
                lblNohayRegistros.Visible = false;
                ListaTerminos.DataBind();

            }
        }

        protected void Aceptar_Click(object sender, EventArgs e)
        {
            
             try
                {
                    ControladorServicio controlador = new ControladorServicio(true);
                    BindingList<Termino> terminos;
                    if (this.txtTermino.Text == "")
                    {
                        terminos = controlador.ObtenerTerminos();
                    }
                    else
                    {
                        terminos = controlador.ObtenerTerminosPorLike(this.txtTermino.Text);

                    }
                    //Guardo la ultima busqueda y el menu activo en la pagina
                    Session["UltimaBusqueda"] = this.txtTermino.Text;
                    vengoDe =(String) ViewState["vengoDe"] ;
                    evaluarVisibles(terminos, controlador);
                    //libero los recursos
                    controlador.Dispose();
                }
             catch (Exception ex)
             {
                 Trace.Write(ErrorConfig.ErrorAplicacion, ex.Message, ex);
                 MsgError.Visible = true;
             }

        }
    }
}
